/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MATHUTILS__
#define __MATHUTILS__

#include <cstdlib>
#include <cmath>

#define PI 3.14159

inline double ABS(double x)
{
	return x < 0.0 ? -x : x;
}

inline bool SignSame(double x, double y)
{
	if( x < 0.0 && y < 0.0 )
		return true;
	else if( x > 0.0 && y > 0.0 )
		return true;
	else // also count either one as 0 as false.
		return false;
}

inline double Sign(double x)
{
	if( x < 0.0 )
		return -1.0;
	else if( x > 0.0 )
		return 1.0;
	else 
		return 0.0; // x is 0
}

template<class constrain_type> inline constrain_type CLIP(constrain_type x, constrain_type min, constrain_type max)
{
	if( x < min )
		return min;
	else if( x > max )
		return max;
	else 
		return x;
}

inline double SQUARE(double x)
{
	return x*x;
}


inline double MIN(double x,double y)
{
	return x < y ? x : y;
}
inline double MAX(double x, double y)
{
	return x > y ? x : y;
}

inline double SafeLog(double x)
{
	return x != 0.0 ? log(x) : x;
}

inline long RandLongMax(long max)
{
	return rand() % max;
}

inline bool RandomBool(void)
{
	return rand() % 2 == 1;
}

inline double Random1(void)
{
	return ((double) rand() / (double) RAND_MAX);
}

inline double RandomN1To1(void)
{
	return (Random1() * 2.0) - 1.0;
}

// Returns a normally distributed deviate with zero mean and unit variance.
double GaussianRandom(void);

inline double Perturb(double value, double maxP)
{
	// perturb a value up to maxP in either direction
	return value + RandomN1To1() * maxP;
}


inline double XAverage(double inAlpha, double inOldAverage,double inNewValue)
{
	return inAlpha * inNewValue + (1.0-inAlpha) * inOldAverage;
}

inline double RoundDownToIncrement(double inValue, double inInc)
{
	long mult = (long)(inValue / inInc);
	double result = inInc * (double)mult;
	return result;
}

inline double RoundUpToIncrement(double inValue, double inInc)
{
	long mult = (long)(inValue / inInc);
	double result = inInc * (double)mult;
	if( result < inValue )
		result += inInc;
	return result;
}

#endif
