/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __MULTIPOPULATION__
#define __MULTIPOPULATION__

#ifndef __POPULATION__
#include "population.h"
#endif

#include <vector>

class MultiPopulation {
public:
	MultiPopulation( void );
	virtual ~MultiPopulation();

	// add populations, must have same blueprint for genomes, but
	// population parameters can be different.
	// once you call this, MultiPopulation object owns inPop even if this throws error.
	void AddPopulation(Population *inPop);

	unsigned GetNumPop(void) const { return mPop.size(); }

	Population *GetPop(unsigned index) { return mPop[index]; }
	const Population *GetPop(unsigned index) const { return mPop[index]; }

	// sets cycles per call of Evolve(),
	// how many times mPop[i]->Evolve is called for each MultiPopulation::Evolve
	void SetCycles(unsigned inCycles) { mCycles = inCycles; }
	unsigned GetCycles(void) const { return mCycles; }


	// crosspolinate between populations, then evolve each population mCycles times
	void Evolve(void);

	// Get winner from among all populations.
	const Genome *GetWinner(void) const;

protected:
	void Reset(void);

	void CrossPollinate(void);

protected:
	std::vector<Population *> mPop; // populations
	unsigned mCycles;
};


#endif
