/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton@ranton.org

For custom software development and consulting please visit http://www.madlogic.com .

Copyright (c) 2003, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __GENOME__
#define __GENOME__

#ifndef __GENOMEBLUEPRINT__
#include "genomeblueprint.h"
#endif

class Genome {
public:
	Genome( GenomeBlueprint *inBlueprint );
	virtual ~Genome(void);

	///// overide these to implement a Genome
	// return a bare (unspecified) specimen, this is use to create initial population,
	// and new random members of population
	virtual Genome *Spawn(void);
	// make this one random
	virtual void Random(void);
	// make this one all zero
	virtual void Zero(void);
	// make an exact copy
	virtual Genome *Copy(void);
	// crossover with another and return the new one
	virtual Genome *Crossover(Genome *inG);
	// mutate this one, meaning (possibly) change it slightly.
	virtual void Mutate(void);
	// return fitness of this specimen
	virtual double Fitness(void);
	
	// this you don't override
	void CalcFitness(void);
	double GetFitness(void) const { return mFitness; }

	GenomeBlueprint *GetBlueprint(void) { return mBlueprint; }

	const Gene& GetGene(unsigned index) const { return mGene[index]; }

	const Gene *GetGene(void) const { return mGene; }

protected:
	void Copy(const Genome& inGenome);

	// get random value for a parameter
	Gene UniformMutate(Gene min, Gene max);
	// get random value with Gaussian distribution centered around previous value.
	Gene GaussianMutate( Gene value, Gene min, Gene max );
	// set to either min or max randomly.
	Gene BoundaryMutate(Gene min, Gene max);

	// do a 2 point simple crossover
	void SimpleCrossover(Genome *child, Genome *mother, Genome *father);
	// do an arithmetic crossover, that is average genes
	void ArithmeticCrossover(Genome *child, Genome *mother, Genome *father);
	// do a heuristic crossover which projects in direction of better parent past better parent.
	void HeuristicCrossover(Genome *child, Genome *mother, Genome *father);

	Gene AverageGenes(Gene a, Gene b);
	Gene ProjectGene(
		Gene better, Gene worse, 
		Gene min, Gene max,
		double vlen );

protected:
	GenomeBlueprint *mBlueprint; // types and ranges of genes.
	Gene *mGene; // genes
	double mFitness; // fitness after evaluation
};


#endif
